package com.miaxis.pc.controller.teachingDsp;

import com.miaxis.common.annotation.Log;
import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.common.enums.BusinessTypeEnum;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.utils.StringUtils;
import com.miaxis.teachingDsp.domain.TeachingDspInfo;
import com.miaxis.teachingDsp.dto.TeachingDspInfoDto;
import com.miaxis.teachingDsp.service.ITeachingDspInfoService;
import com.miaxis.teachingDsp.vo.TeachingDspInfoVo;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 【学车短视频】Controller
 *
 * @author miaxis
 * @date 2021-12-09
 */
@RestController
@AllArgsConstructor
@RequestMapping("/pc/teachingDsp/dsp")
@Api(tags={"【pc-学车短视频】"})
public class TeachingDspInfoController extends BaseController{
    @Autowired
    private ITeachingDspInfoService teachingDspInfoService;

    /**
     * 查询学车短视频列表
     */
    @PreAuthorize("@ss.hasPermi('teachingDsp:dsp:list')")
    @GetMapping("/list")
    @ApiOperation("查询学车短视频列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<TeachingDspInfoVo> list(@ModelAttribute TeachingDspInfo teachingDspInfo){
        startPage();
        List<TeachingDspInfoVo> list = teachingDspInfoService.selectTeachingDspInfoList(teachingDspInfo);
        return toResponsePageInfo(list);
    }





    /**
     * 新增学车短视频
     */
    @PreAuthorize("@ss.hasPermi('teachingDsp:info:add')")
    @Log(title = "新增学车短视频", businessType = BusinessTypeEnum.INSERT)
    @PostMapping
    @ApiOperation("新增教学视频")
    public Response add(@RequestBody TeachingDspInfoDto teachingDspInfoDto){
        return teachingDspInfoService.saveTeachingDsp(teachingDspInfoDto);
    }


    /**
     * 修改学车短视频
     */
    @PreAuthorize("@ss.hasPermi('teachingDsp:info:edit')")
    @Log(title = "修改学车短视频", businessType = BusinessTypeEnum.UPDATE)
    @PutMapping
    @ApiOperation("修改学车短视频")
    public Response edit(@RequestBody TeachingDspInfoDto teachingDspInfoDto){
        return teachingDspInfoService.updateTeachingDspById(teachingDspInfoDto);
    }


    /**
     * 删除学车短视频

    @PreAuthorize("@ss.hasPermi('teachingDsp:dsp:remove')")
    @Log(title = "学车短视频", businessType = BusinessTypeEnum.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除学车短视频")
    public  Response<Integer> remove(
            @ApiParam(name = "ids", value = "学车短视频ids参数", required = true)
            @PathVariable Long[] ids
    ){
        return toResponse(teachingDspInfoService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
     */

    /**
     * 删除教学视频
     */
    @PreAuthorize("@ss.hasPermi('teachingVideo:info:remove')")
    @Log(title = "教学视频", businessType = BusinessTypeEnum.UPDATE)
    @PutMapping("/{ids}")
    @ApiOperation("删除教学视频")
    public  Response remove(
            @ApiParam(name = "ids", value = "教学视频ids参数", required = true)
            @PathVariable Long[] ids
    ){
        return teachingDspInfoService.removeTeachingDspByIds(ids);
    }

    /**
     * 上架短视频
     */
    @PreAuthorize("@ss.hasPermi('teachingVideo:info:putShelf')")
    @Log(title = "上架短视频", businessType = BusinessTypeEnum.UPDATE)
    @PutMapping("/putShelf/{ids}")
    @ApiOperation("上架短视频")
    public  Response putShelf(
            @ApiParam(name = "ids", value = "教学视频ids参数", required = true) @PathVariable Long[] ids
    ){
        return teachingDspInfoService.removePutShelfByIds(ids);
    }

    /**
     * 下架短视频
     */
    @PreAuthorize("@ss.hasPermi('teachingVideo:info:offShelf')")
    @Log(title = "下架短视频", businessType = BusinessTypeEnum.UPDATE)
    @PutMapping("/offShelf/{ids}")
    @ApiOperation("下架短视频")
    public  Response offShelf(
            @ApiParam(name = "ids", value = "教学视频ids参数", required = true) @PathVariable Long[] ids
    ){
        return teachingDspInfoService.removeOffShelfByIds(ids);
    }


    /**
     * 短视频封面上传
     */
    @Log(title = "短视频封面上传", businessType = BusinessTypeEnum.INSERT)
    @PutMapping("/coverUp")
    @ApiOperation("短视频封面上传")
    public Response coverUp(MultipartFile coverFile,
                            @ApiParam(name = "fileId",value = "云点播fileId",required = true) @RequestParam("fileId") String fileId){
        if (StringUtils.isEmpty(fileId)){
            throw new CustomException("视频文件未上传");
        }
        return teachingDspInfoService.coverUp(fileId,coverFile);
    }

    /**
     * 获取短视频详细信息
     */
    @PreAuthorize("@ss.hasPermi('teachingDsp:info:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation("获取短视频详细信息")
    public Response<TeachingDspInfoVo> getInfo(
            @ApiParam(name = "id", value = "获取短视频参数", required = true)
            @PathVariable("id") Long id
    ){
        return teachingDspInfoService.getTeachingDspDetailsById(id);
    }

}
